/*
 * @author Georg Kompacher, member of the ISAC-team, 
 * Copyright (c) 2006 by Georg Kompacher
 * created Jul 10, 2006 04:18:01 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.browserdialog;

import isac.gui.util.HierarchyNode;
import isac.interfaces.IToGUI;
import isac.session.Session;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UserActionOnLink;
import isac.useractions.UserActionOnMathContext;
import isac.util.formulae.Context;
import isac.util.formulae.ContextUnknown;
import isac.util.formulae.HierarchyKey;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IContextProvider;
import isac.wsdialog.WorksheetState;
import isac.wsdialog.IContextProvider.ContextType;

import java.net.URL;
import java.rmi.RemoteException;
import java.util.Random;

import javax.swing.tree.TreePath;

/**
 * @author Georg Kompacher
 * 
 * This class is responsible for the interactions to the ProblemBrowser, to the
 * active WSDialog, and also to other BrowserDialogs.
 */
public class ProblemDialog extends BrowserDialog {

	/**
	 * For serialization, see corresponding API Documentation; value generated
	 * by eclipse to avoid a warning
	 */
	private static final long serialVersionUID = 6479711601158544388L;

	/** true if the refine button is visible * */
	private boolean refine_visible_;

	/** true if the to worksheet button is visible * */
	private boolean to_worksheet_visible_;

	/** true if the Context On button is visible* */
	//private boolean context_on_visible_;
	
	/** true if a worksheet is open
	 *  this flag is updated in updateState()
	 */
	private boolean worksheet_open_;

	/** true if the calc head of the active worksheet is open
	 *  this flag is updated in updateState()
	 */
	private boolean calc_head_open_;

	/** true if the context updating is on * */
	//private boolean context_on_;

	/**
	 * The constructor of this class which is given the session the
	 * ProblemDialog is instanciated for. We need the session to get the correct
	 * WSDialogManager (and also the active WSDialog) for this BrowserDialog.
	 * 
	 * @param session
	 * @throws RemoteException
	 */
	public ProblemDialog(Session session) throws RemoteException {
		super(session);
		context_type_ = ContextType.CONTEXT_PROBLEMS;
		
		hierarchy_ = loadHierarchy(context_type_);
		current_context_ = new ContextUnknown();
	}

	/**
	 * Notify the ProblemDialog about events or requests triggered by the user.
	 * The ProblemBrowser calls this method over RMI.
	 * 
	 * @param action
	 *            What's new from the UI
	 * @return true, if the event is being processed, false if processing is
	 *         denied or impossible
	 * @throws RemoteException
	 * @throws DialogProtocolException
	 */
	public boolean notifyUserAction(IUserAction action)
			throws DialogProtocolException, RemoteException {
		System.out.println("ProblemDialog::notifyUserAction() action="
				+ action.getActionID());
		
		if( super.notifyUserAction( action ) )
			return true;
		
		switch (action.getActionID()) {
		case UI_BROWSER_MATCH_ON: {
			context_on_ = true;
			drawButtons();
			sendInitContextToBrowser();
			return true;
		}
		case UI_BROWSER_MATCH_OFF: {
			context_on_ = false;
			drawButtons();
			ContextUnknown new_context= new ContextUnknown();
			new_context.setHierarchyKey(current_context_.getHierarchyKey());
			new_context.setKEStoreKey(current_context_.getKEStoreKey());
			sendContextToBrowser(new_context);
			sendLinkToMiniBrowser(new_context,context_type_);
			return true;
		}
		case UI_PROBLEMBROWSER_REFINE: {
			sendRefinedCurrentContext();
			return true;
		}
		case UI_BROWSER_TO_WORKSHEET: {
			setCurrentContextToWorksheet();
			return true;
		}
		case UI_MINIBROWSER_LINK: {
			URL receivedLink = ((UserActionOnLink) action).getLink();
			try {
				interpretLink(receivedLink);
				return true;
			} catch (RemoteException e) {
				e.printStackTrace();
				return false;
			}
		}
		}
		return false;
	}


	/**
	 * removes the button for Refine. If the button does not
	 * exist so far, nothing happens.
	 */
	private void removeButtonRefine() {
		if (!refine_visible_)
			return;
		try {
			browser_frame_rmi_.removeUIElement(new UIAction(
					EUIElement.UI_PROBLEMBROWSER_REFINE,
					EUIContext.UI_CONTEXT_BROWSER, user_language_));
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		refine_visible_ = false;
	}

	/**
	 * creates the button for Refine. If the button does already
	 * exist, nothing happens.
	 */
	private void drawButtonRefine() {
		if (refine_visible_)
			return;
		try {
			browser_frame_rmi_.addUIElement(new UIAction(
					EUIElement.UI_PROBLEMBROWSER_REFINE,
					EUIContext.UI_CONTEXT_BROWSER, user_language_));
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		refine_visible_ = true;
	}

	/**
	 * removes the button for To Worksheet. If the button does not
	 * exist so far, nothing happens.
	 */
	private void removeButtonToWorksheet() {
		if (!to_worksheet_visible_)
			return;
		try {
			browser_frame_rmi_.removeUIElement(new UIAction(
					EUIElement.UI_BROWSER_TO_WORKSHEET,
					EUIContext.UI_CONTEXT_BROWSER, user_language_));
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		to_worksheet_visible_ = false;
	}

	/**
	 * generates the button for To Worksheet. If the button does 
	 * already exist, nothing happens.
	 */
	private void drawButtonToWorksheet() {
		if (to_worksheet_visible_)
			return;
		try {
			browser_frame_rmi_.addUIElement(new UIAction(
					EUIElement.UI_BROWSER_TO_WORKSHEET,
					EUIContext.UI_CONTEXT_BROWSER, user_language_));
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		to_worksheet_visible_ = true;
	}

	/**
	 * This method overrides the method in the base class and
	 * calls the method of the base class. This is done, as some
	 * additional initialisations have to be done with the buttons
	 * and the context.
	 * 
	 * @param browser_frame
	 *            the browser frame to register
	 */
	public void registerBrowserFrame(IToGUI browser_frame)
			throws RemoteException {
		super.registerBrowserFrame(browser_frame);
		initButtons();
	}
	
	
	private void initButtons() {
		context_on_visible_ = false;
		to_worksheet_visible_ = false;
		refine_visible_ = false;
		context_on_ = true;
		try {
			browser_frame_rmi_.addUIElement(new UIAction(
					EUIElement.UI_BROWSER_MATCH_OFF,
					EUIContext.UI_CONTEXT_BROWSER, user_language_));
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
	
	
	
	
	
	
	

	/**
	 * This method draws the buttons of the browser. Which buttons need
	 * to be shown depends on three flags:
	 * context_on_   worksheet_open_   calc_head_open_  | Buttons to show:
	 * -----------------------------------------------------------------
	 *    F               X                X          | <Context On>		
	 *    T               F                F          | <Context Off>	
	 *    T               F                T          | impossible	
	 *    T               T                F          | <Context Off> <Refine>
	 *    T               T                T          | <Context Off> <Refine> <ToWorksheet>			  
	 *    
	 */

	protected void drawButtons() {
		if (!context_on_) {
			drawButtonContextOn();
			removeButtonRefine();
			removeButtonToWorksheet();
		} else {
			if (!worksheet_open_ && !calc_head_open_) {
				drawButtonContextOff();
				removeButtonRefine();
				removeButtonToWorksheet();
			}
			if (worksheet_open_ && !calc_head_open_) {
				drawButtonContextOff();
				drawButtonRefine();
				removeButtonToWorksheet();
			}
			if (worksheet_open_ && calc_head_open_) {
				drawButtonContextOff();
				drawButtonRefine();
				drawButtonToWorksheet();
			}
		}
	}
	/**
	 * This method does the update for the two variables:
	 * worksheet_open_ and calc_head_open_. An interface to the
	 * active worksheet is fetched from the session. If no active
	 * worksheet exists (and null is returned) there is no open
	 * worksheet. The interface returns a WorksheetState which is
	 * able to tell us, if the CalcHead is open.
	 */
	public void updateState() {

		 try {
		 IContextProvider context_provider = session_.getContextProvider();
		 if(context_provider == null) {
		 worksheet_open_ =  false;
		 calc_head_open_ = false;
		 return;
		 }
		 worksheet_open_ =  true;
		 WorksheetState state = context_provider.getWorksheetState();
		 calc_head_open_ = state.isCalcHeadOpen();
		 }catch(RemoteException e) {
		 e.printStackTrace();
		 }
	}
	


	/**
	 * This method takes the current context and asks the active Worksheet
	 * to refine it. This is done over the IContextProvider interface.
	 * The active Worksheet returns a refined context, which is then
	 * shown in the browser. There must be a ContextProvider when this
	 * method is called. If there would not be any active worksheet, the
	 * button <Refine> would not be available.
	 */
	private void sendRefinedCurrentContext() {
		//real implementation:
		
		 Context refined_context = current_context_;
		 try {
		 IContextProvider context_provider = session_.getContextProvider();
		 if(context_provider == null)
		 return;
		 refined_context = context_provider.refineContext(current_context_);
		 sendContextToBrowser(refined_context);
		 sendLinkToMiniBrowser(refined_context,context_type_);
		 }catch(RemoteException e) {
		 e.printStackTrace();
		 }

		//		dummy implementation for test purposes while initContext() is not
		//implemented in WorksheetDialog:
		/*
		HierarchyKey item_to_select = new HierarchyKey();

		item_to_select.addString("rat");
		item_to_select.addString("sq");
		item_to_select.addString("root");
		item_to_select.addString("univariate");
		item_to_select.addString("equation");

		TreePath tree_path = hierarchy_.getPath(item_to_select.getID());
		HierarchyNode node_to_select = (HierarchyNode) tree_path
				.getLastPathComponent();
		Random a = new Random();
		Context context = null;
		int rand = a.nextInt(5);
		if (rand > 2) {
			context = current_context_;
		} else {
			context = new Context();
			context.setHierarchyKey(item_to_select);
			context.setHierarchyNode(node_to_select);
		}
		sendContextToBrowser(context);
		sendLinkToMiniBrowser(context,context_type_);
		*/
		//end Experimantation:	
	}

}
