package isac.gui.browser.panels;

import isac.gui.IGuiConstants;
import isac.gui.browser.BrowserFrame;
import isac.gui.browser.BrowserTreeModel;
import isac.gui.browser.IContextChangeListener;
import isac.gui.util.Hierarchy;
import isac.gui.util.HierarchyNode;

import isac.interfaces.IToGUI;

import isac.useractions.EUIContext;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIActionOnHierarchy;
import isac.util.formulae.Context;
import isac.util.formulae.HierarchyKey;

import java.awt.BorderLayout;
import java.awt.Dimension;

import java.rmi.RemoteException;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JViewport;

import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/***
 * The class HierarchyPanel holds the graphic representation of the tree structure
 * The Swing class JTree is used for this representation.
 * 
 *  Additionaly the HierarchyPanel can process the UIActions that come from the 
 *  BrowserDialog if they have the UI_CONTEXT_HIERARCHY context  
 *   
 */
public class HierarchyPanel extends JPanel implements IToGUI, IContextChangeListener {

	static final long serialVersionUID = -3662045714502512897L;
	
	/**
	 * The parent BrowserFrame is added as a MouseListener to the 
	 * tree. I belive that the children should know their parents.
	 */
	private BrowserFrame parent_;
	
	/**
	 * The actual graphic representation. Relies heavily on the TreeModel (see below).
	 */
	private JTree view_tree_;
	
	/**
	 * The abstraction level required for the JTree structure. 
	 * A TreeModel epresents a data structure (tree) of Objects (!= null).
	 * 
	 * As a static structure, a TreeModel is a (connected) directed graph which is acyclic 
	 * with a high probability and which has a total order on all edges that leave an element, 
	 * a boolean function "leaf" on all nodes that only have incoming edges, and a "root" element; 
	 * by starting there one can reach all other elements. 
	 * 
	 * As a dynamic structure, that concept doesn't work.
	 * 
	 * In order to be able to work properly with TreeModels (and 
	 * because there are places where the current JTree implementation 
	 * requires it), the following constraints should be imposed 
	 * on the objects in a TreeModel :
	 *
	 * 1. Node comparison is done by equals(), not by reference comparison
	 * 2. Node objects may not "change" dynamically with time
	 * 3. null may not be contained in a tree model
	 * 
	 * The locations in a TreeModel are represented by TreePaths.
	 * TreePath is implemented as a singly-linked list that can be walked 
	 * up towards the root. It is immutable.
	 * A Node in TreeModel can have multiple paths (in ISAC hierarchy that is not the case).
	 *   
	 */	
	protected BrowserTreeModel 	model_;
	
	protected Hierarchy 	hierarchy_;
		
	/**
	 * Constructor creates all necessary GUI components and sets up the listeners.
	 * 
	 * @param parent - added to the JTree as a MouseListener
	 * @param hierarchy - data used to build the tree
	 */
	public HierarchyPanel( BrowserFrame parent ) {
		parent_ 	= parent;
		hierarchy_ 	= new Hierarchy();
		
		view_tree_ = null;
		initTree();
		
		parent_.addContextChangeListener( this );
	}

	private void initTree() {
		model_ 	   = new BrowserTreeModel(hierarchy_);
        view_tree_ = new JTree(model_);
        view_tree_.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        view_tree_.setExpandsSelectedPaths( true );
        view_tree_.setScrollsOnExpand( true );
        view_tree_.setRootVisible(true);
        
        view_tree_.addMouseListener( new HierarchyPanelListener( parent_ ) );
        setLayout(new BorderLayout());
                
        add(new JScrollPane(view_tree_),BorderLayout.CENTER);
        
        this.setMinimumSize( 
    			new Dimension( 
    				IGuiConstants.HIERARCHY_MIN_SIZE_X, 
    				IGuiConstants.HIERARCHY_MIN_SIZE_Y 
    			)
    		);        
    }
	
	/**
	 * 
	 * Method inherited from IToGUI interface, but not really implemented here.
	 * 
	 */
	public void addUIElement(IUIAction action) throws RemoteException {
		System.out.println("In HierarchyPanel::addUIElement(IUIAction a) : Action is " + action.getAction() + 
				", Context is : " + action.getContext() );
		System.exit(1);		
	}

	/**
	 * 
	 * Method inherited from IToGUI interface.
	 * 
	 * Performs the UI actions that have UI_CONTEXT_HIERARCHY context.
	 * Mainly for seting the selection in the tree.
	 * 
	 */		
	public void doUIAction(IUIAction action) throws RemoteException {
		System.out.println("In HierarchyPanel::doUIAction(IUIAction a) : Action is " + action.getAction() + 
				", Context is : " + action.getContext() );
		
		if( action.getContext() != EUIContext.UI_CONTEXT_HIERARCHY )
			return;
		
		switch( action.getAction() ) {
		case UI_BROWSER_HIERARCHY: 
			UIActionOnHierarchy uaoh = (UIActionOnHierarchy) action;
			hierarchy_ = uaoh.getHierarchy();
			remove( view_tree_ );
			initTree();
			break;
		default:
			System.out.println("Which witch watches which switch (in HierarchyPanel::doUIAction()");
			break;
		}
		
		return;
	}

	/**
	 * 
	 * Method inherited from IToGUI interface, but not really implemented here.
	 * 
	 */
	public boolean doUIAction(IUserAction action) throws RemoteException {
		System.out.println("In HierarchyPanel::doUIAction(IUserAction a) : Action is " + action.getActionID() + 
				", Context is : " + action.getContext() );
		System.exit(1);
		return false;
	}

	/**
	 * Method inherited from IToGUI interface, but not really implemented here.
	 */
	public boolean removeUIElement(IUIAction action) throws RemoteException {
		System.out.println("In HierarchyPanel::removeUIElement(IUIAction a) : Action is " + action.getAction() + 
				", Context is : " + action.getContext() );
		System.exit(1);			
		return false;
	}

	/* (non-Javadoc)
	 * @see isac.gui.browser.IContextChangeListener#contextChanged(isac.util.formulae.Context)
	 */
	public void contextChanged(Context new_context) {
		System.out.println("In HierarchyPanel::contextChanged(Context) : HierarchyKey is " + new_context.getHierarchyKey() );
		HierarchyKey key = new_context.getHierarchyKey();
		
		if( key == null )
			return;

		HierarchyNode node = hierarchy_.getNodeForHierarchyKey( key.getID() );
		
		TreePath path = model_.getPath(node);
		
		System.out.println("In HierarchyPanel::contextChanged(Context) : TreePath is " + path );
		
		if( path == null || path.getLastPathComponent() == null )
			return;
		
		view_tree_.setSelectionPath(path);
		view_tree_.updateUI();
		
		((JViewport)view_tree_.getParent()).scrollRectToVisible(view_tree_.getPathBounds( path ));
	}

	public boolean isUIRunning() throws RemoteException {
		return true;
	}

}