/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.treetable;

import isac.gui.Worksheet;
import isac.gui.WorksheetPopupHandler;
import isac.interfaces.ICalcElement;
import isac.interfaces.ICalcIterator;
import isac.useractions.IUIAction;
import isac.useractions.UIActionOnCalcElement;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.CalcHead;
import isac.util.formulae.Formula;
import isac.util.formulae.Position;
import isac.util.tactics.Tactic;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JPopupMenu;

import org.apache.log4j.Logger;

/**
 * This class is responsible for the representation of a particular
 * <code>CalcElement</code> in a <code>CalcModelHierarchy</code>
 * 
 * @author Mario Hochreiter 2004
 * @version 1.0
 * @author Alois Kirchsteiger 2005
 * @version 1.1
 */
public class CalcModelNode implements Serializable {
    static Logger logger = Logger.getLogger(CalcModelNode.class.getName());

    static final long serialVersionUID = -3885058116359064437L;

	/**
     * @deprecated the formula is now stored in elem_
     */ 
    protected ICalcIterator it_;//for fetching the formula in this node
    
    protected ICalcElement elem_;

    protected Vector<CalcModelNode> children_;
    
    protected CalcModelNode parent_;

    protected String tacticString_ = "";
    protected String assumptionString_ = "";
    
    protected WorksheetPopupHandler formulaPopupHandler_ = null;
    protected WorksheetPopupHandler tacticPopupHandler_ = null;
    
    protected int showWhat_ = 0;
    
    protected final int SHOW_TACTIC = 0;
    protected final int SHOW_ASSUMPTION = 1;
    
    public CalcModelNode(ICalcIterator it) {
        it_ = null;
        children_ = new Vector<CalcModelNode>();
        parent_ = null;
        try {
        	elem_ = null;
        	if( it != null )
        		elem_ = it.getElement();
        } catch(RemoteException re) {
        	re.printStackTrace();
        }
        
        formulaPopupHandler_ = new WorksheetPopupHandler();
        tacticPopupHandler_ = new WorksheetPopupHandler();
    }
    
    public CalcModelNode( ICalcElement elem ) {
    	it_ = null;
    	children_ = new Vector<CalcModelNode>();
    	parent_ = null;
    	elem_ = elem;
    	
        formulaPopupHandler_ = new WorksheetPopupHandler();
        tacticPopupHandler_ = new WorksheetPopupHandler();
    }

    public JPopupMenu getFormulaContextMenu( Worksheet ws ) {
    	return formulaPopupHandler_.buildContextMenu( ws );
    }
    
    public JPopupMenu getFormulaContextMenu( Worksheet ws, JPopupMenu commonActions ) {
    	return formulaPopupHandler_.mergeContextMenu( ws, commonActions );
    }
    
    public JPopupMenu getTacticContextMenu( Worksheet ws ) {
    	return tacticPopupHandler_.buildContextMenu( ws );
    }
   
    public JPopupMenu getTacticContextMenuTest( Worksheet ws, JPopupMenu commonActions ) {
    	return tacticPopupHandler_.mergeContextMenu( ws, commonActions );
    }
    
    public void addDynamicAction( IUIAction action ) {
    	formulaPopupHandler_.addAction( action );
    }
    
    public void addApplicableTactic( IUIAction tactic ) {
    	UIActionOnCalcElement uaoce = (UIActionOnCalcElement) tactic;
    	
    	tacticPopupHandler_.addAction( tactic, ((Tactic)uaoce.getCalcElement()).toSMLString() );
    }
    
    /**
     * 
     * @param child
     * @param index
     * @return true is successful, false if node could not be added at the
     *         specuified position
     */
    public boolean add(int index, CalcModelNode child) {
        try {
            children_.add(index, child);
            child.parent_ = this;
            return true;
        } catch (ArrayIndexOutOfBoundsException e) {
            return false;
        }
    }

    /**
     * inserts the child on the right position, which is determined by the
     * position of the iterator
     * 
     * @param child
     * @return true if successful
     */
    public boolean add(CalcModelNode child) {
        Iterator it = children_.iterator();
        boolean successorFound = false;
        int index = 0;
        while (!successorFound && it.hasNext()) {
            CalcModelNode currentNode = (CalcModelNode) it.next();
            if (currentNode.getPosition() == null) {
                successorFound = true;
            } else { 
            	if ( currentNode.getPosition().compareTo( child.getPosition() ) < 0) {
            		index++;
            	} else if ( currentNode.getPosition().compareTo( child.getPosition() ) == 0 ) {
            		// NC 060225 Replace formula
            		children_.remove( currentNode );
            		successorFound = true;
//            		System.out.println("Positions are same ... Node exists ... ");
//            		return false; //node with the value already exists CalcModelHierarchy.compareIterators(currentNode.getValue(), child.getValue()) == 0
            	} else { // currentNode has a higher position than child
            		successorFound = true;
            	}
            }
        }
      
        children_.add(index, child);
        child.parent_ = this;
        return true;
    }

    /**
     * @return the path to this node, where the root is at index 0 and the
     *         parent node is at index size-1
     */
    public Vector<CalcModelNode> getPath() {
        Vector<CalcModelNode> result = new Vector<CalcModelNode>();
        if (parent_ != null) {
            result = parent_.getPath();
        }
        result.add(this);
        return result;
    }

    /**
     * @return the path to the parent node, where the root is at index 0 and the
     *         parent node is at index size-1
     */
    public Vector getParentPath() {
        Vector result = new Vector();
        if (parent_ != null) {
            result = parent_.getPath();
        }
        return result;
    }

    /**
     * @return the children of this node
     */
    public Vector getChildren() {
        return children_;
    }

    /**
     * @param index
     * @return the child at the specified index
     */
    public CalcModelNode getChild(int index) {
        return (CalcModelNode) children_.get(index);
    }

    /**
     * removes the specified child
     */
    public void remove(CalcModelNode child) {
        child.parent_ = null;
        children_.remove(child);
    }

    /**
     * @return the parent <code>CalcModelNode</code> of this node
     */
    public CalcModelNode getParent() {
        return parent_;
    }

    /**
     * 
     * @return index of this node as seen from its parent
     */
    public int getIndex() {
        return parent_.getChildIndex(this);
    }
    
    /**
     * 
     * @return the next child node or null if there is no next child node
     */
    public CalcModelNode getNextChild( CalcModelNode node ) {
        Iterator it = children_.iterator();
        boolean successorFound = false;

        if( node == null ) {
        	if( it.hasNext() )
        		return (CalcModelNode) it.next();
        	else
        		return null;
        }
        	
        while (!successorFound && it.hasNext()) {
            CalcModelNode currentNode = (CalcModelNode) it.next();
            if ( currentNode.getPosition().compareTo( node.getPosition() ) == 0 ) {
        		successorFound = true;
        	}
        }
    	
        if( successorFound && it.hasNext() )
        	return (CalcModelNode) it.next();
        
        return null;
    }
    
    /**
     * for fetching the next node in the tree hierarchy
     * 
     * @return the next node or null if there is no next node
     */
    public CalcModelNode getNextNode( ) {
    	
    	if( parent_ != null )
    		return parent_.getNextChild( this );
    	
    	return null;
    }
    

    /**
     * return the value of the node, which is a <code>ICalcIterator</code>
     * @deprecated in favor of getFormula() or getPosition()
     */
    public ICalcIterator getValue() {
        return it_;
    }
    
    /**
     * return the Formula attached to the node or null if the element is not a CalcFormula
     */
    public CalcFormula getFormula() {
    	if( elem_ instanceof CalcFormula )
    		return (CalcFormula) elem_;
   		return null;
    }

    public String getSecondaryString() {
    	switch( showWhat_ )
    	{
    	case SHOW_TACTIC:
    		return tacticString_;
    	case SHOW_ASSUMPTION:
    		return assumptionString_;
    	default:
    		return "";	
    	}
    }
    
    public void setTacticString( String tacticString )
    {
    	tacticString_ 	= tacticString;
    	showWhat_ 		= SHOW_TACTIC;
    }
    
    public void setAssumptionString( String assumptionString )
    {
    	assumptionString_ = assumptionString;
    	showWhat_ = SHOW_ASSUMPTION;
    }
    
    /**
     * return the CalcHead attached to the node or null if the element is not a CalcHead
     */    
    public CalcHead getCalcHead() {
    	if( elem_ instanceof CalcHead )
    		return (CalcHead) elem_;
   		return null;
    }
    
    /**
     * @return the ICalcElement of the node
     */
    public ICalcElement getCalcElement() {
    	return elem_;
    }
    
    /**
     * return the Position attached to the node or null if the element has no position
     */    
    public Position getPosition() {
    	CalcFormula cf = getFormula();
    	CalcHead ch = getCalcHead();
    	
    	if( cf != null )
    		return cf.getPosition();
    	
    	if( ch != null )
    		return ch.getPosition();
    	
   		return null;
    }    

    /**
     * @return Visual representation of a <code>CalcElement</code> TOO OFTEN
     *         used by standard Java rendering of Worksheet
     */
    public String toString() {
    	if (elem_ != null) {
            try {
                logger.fatal(" WS->DG: toString>getFormula");
                ICalcElement element = elem_;
                logger.fatal(" WS<-DG: element=" + element.toSMLString());
                if (element instanceof CalcFormula) {
                    return ((CalcFormula) element).toString();
                } else if (element instanceof CalcHead) {
                    return ((Formula) ((CalcHead) element).getHeadLine())
                            .toSMLString();
                } else {
                    return null;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return "    ";
    }

    /**
     * compares a CalcModelNode to a object if this object is also a
     * CalcModelNode an its CalcElement is the same, this method will return
     * true
     */
    public boolean equals(Object o) {
        if (o instanceof CalcModelNode) {
            CalcModelNode node = (CalcModelNode) o;
            
            Position old_pos = node.getPosition();
            Position my_pos = getPosition();
            
            if (old_pos != null && my_pos != null) {
                return (my_pos.compareTo(old_pos) == 0);
            } else {
                return (old_pos == null && my_pos == null); //this.ce equals
                // node.ce which is
                // also null
            }
        }
        return false;
    }

    /**
     * @return
     */
    public int getChildCount() {
        return children_.size();
    }

    /**
     * @param child
     * @return index of the child
     */
    public int getChildIndex(CalcModelNode child) {       	
        return children_.indexOf(child);
    }
}