/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.treetable;

import isac.gui.IGuiConstants;
import isac.gui.Worksheet;

import java.awt.*;
import java.io.*;

import javax.swing.*;
import javax.swing.plaf.ColorUIResource;
import javax.swing.tree.TreeCellRenderer;

/**
 * Custom Tree Cell Renderer
 * @version 1.0
 * @author Mario Hochreiter
 */
public class CustomTreeCellRenderer
  extends JPanel
  implements TreeCellRenderer {
  
  static final long serialVersionUID = 6590921149518486656L;
  
  protected Worksheet worksheet_;
  
  private JLabel image;
  private TreeTextPanel text;

  public CustomTreeCellRenderer( Worksheet ws ) {
    setLayout(new BorderLayout());
    image = new JLabel();
    text = new TreeTextPanel();
    add(image, BorderLayout.WEST);
    add(text, BorderLayout.CENTER);
    setSize(5, 30);
    text.setOpaque(true);
    image.setOpaque(true);
    this.setOpaque(true);
    worksheet_ = ws;
  }

  public Component getTreeCellRendererComponent(
    JTree tree,
    Object value,
    boolean selected,
    boolean expanded,
    boolean leaf,
    int row,
    boolean hasFocus) {

    String stringValue =
        tree.convertValueToText(value, selected, expanded, leaf, row, hasFocus);
	try {
	    text.setTacticText(((CalcModelNode)value).getSecondaryString() + "  ");
	} catch (Exception e) {
	  text.setTacticText("");
	}
    text.setFormulaText(stringValue + "  ");
    text.setSelect(selected);
    text.setFocus(hasFocus);
    
    if (selected) {
      setForeground(UIManager.getColor("Tree.selectionForeground"));
    } else
      setForeground(UIManager.getColor("Tree.Foreground"));
    // There needs to be a way to specify disabled icons.
    if (!tree.isEnabled()) {
      setEnabled(false);
      if (leaf) {
        setDisabledIcon(UIManager.getIcon("Tree.leafIcon"));
      } else if (expanded) {
        setDisabledIcon(UIManager.getIcon("Tree.openIcon"));
      } else {
        setDisabledIcon(UIManager.getIcon("Tree.closedIcon"));
      }
    } else {
      setEnabled(true);
      if (leaf) {
        setIcon(UIManager.getIcon("Tree.leafIcon"));
      } else if (expanded) {
        setIcon(UIManager.getIcon("Tree.openIcon"));
      } else {
        setIcon(UIManager.getIcon("Tree.closedIcon"));
      }
    }
    
    setComponentOrientation(tree.getComponentOrientation());
    //this.setBackground(UIManager.getColor("Table.Foreground"));
    this.setPreferredSize(getPreferredSize());
    return this;
  }

  public Dimension getSize() {
    Dimension iconD = image.getSize();
    Dimension textD = text.getPreferredSize();
    int height = iconD.height < textD.height ? textD.height : iconD.height;
    Rectangle r = worksheet_.getBounds();
    int width = r.width == 0 ? IGuiConstants.WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X - 50 : r.width - 50;
    //return new Dimension(iconD.width + textD.width, height);
    return new Dimension(width, height);
  }
  
  public Dimension getPreferredSize() {
    return getSize();
  }
  

  public void setDisabledIcon(Icon icon) {
    image.setIcon(icon);   
  }

  public void setIcon(Icon icon) {
    image.setIcon(icon);    
  }
  
  public FormulaTextField getFormulaField() {
	  return text.getFormulaTextField();
  }
  
  public TacticTextField getTacticField() {
	  return text.getTacticTextField();
  }  
  
  public JPanel getTreeTextPanel() {
	  return text;
  }
}

class TreeTextArea extends JTextArea {
  static final long serialVersionUID = 5536513715221108656L;

Dimension preferredSize;

  TreeTextArea() {
    setLineWrap(true);
    setWrapStyleWord(true);
    setOpaque(true);
  }

  public void setBackground(Color color) {
    if (color instanceof ColorUIResource)
      color = null;
    super.setBackground(color);
  }

  public void setSize(Dimension d) {
    if (d != null) {
      preferredSize = d;
    }
  }

  public Dimension getSize() {
    return preferredSize;
  }
  
  public Dimension getPreferredSize() {
    return preferredSize;
  }

  public void setText(String str) {
    BufferedReader br = new BufferedReader(new StringReader(str));
    FontMetrics fm = this.getFontMetrics(this.getFont());
    String line;
    int lines = 0;
    int maxWidth = 0;
    try {
      while ((line = br.readLine()) != null) {
        int width = SwingUtilities.computeStringWidth(fm, line);
        if (maxWidth < width) {
          maxWidth = width;
        }
        lines++;
      }
    } catch (IOException ex) {
      ex.printStackTrace();
    }
    lines = (lines < 1) ? 1 : lines;
    int height = fm.getHeight() * lines;
    setSize(new Dimension(maxWidth + 6, height));
    super.setText(str);
  }
}

class TreeTextPanel extends JPanel {
  static final long serialVersionUID = -195808786451707391L;

  private FormulaTextField formulaText;
  private TacticTextField tacticText;

  TreeTextPanel() {
	   
    tacticText = new TacticTextField();
    formulaText = new FormulaTextField();//TreeTextArea();
    
    formulaText.setOpaque(true);
    
    tacticText.setOpaque(true);
    tacticText.setHorizontalAlignment(JTextField.RIGHT);
    tacticText.setBorder(null);
    
    this.setOpaque(true);
    this.setBackground(UIManager.getColor("Tree.background"));
    
    GridBagLayout gbl = new GridBagLayout();
    this.setLayout(gbl);
    
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
        
    gbc.anchor = GridBagConstraints.WEST;
    gbl.setConstraints( formulaText, gbc );
    this.add(formulaText);
    
    gbc.anchor = GridBagConstraints.EAST;
    gbc.insets = new Insets( 0, 30, 0, 0 );
    gbl.setConstraints( tacticText, gbc );
    this.add(tacticText);
  }
  
  void setFormulaText(String s) {
    formulaText.setText(s);  
    this.repaint();
  }
  
  void setTacticText(String s) {
    tacticText.setText(s);
    this.repaint();
  }
  
  FormulaTextField getFormulaTextField() {
	  return formulaText;
  }
  
  TacticTextField getTacticTextField() {
	  return tacticText;
  }
  
  void setSelect(boolean isSelected) {
    Color bColor;
    if (isSelected) {
      bColor = UIManager.getColor("Tree.selectionBackground");
    } else {
      bColor = UIManager.getColor("Tree.textBackground");
    }
    super.setBackground(bColor);
  }
  
  public Dimension getPreferredSize() {
    int widthTactic = tacticText.getPreferredSize().width + 30;
    int widthFormula = formulaText.getPreferredSize().width;
    int heightTactic = tacticText.getPreferredSize().height;
    int heightFormula = formulaText.getPreferredSize().height;
    int newWidth = widthTactic > widthFormula ? widthTactic : widthFormula;
    return new Dimension(newWidth + 10, heightTactic + heightFormula);   
  }
  
  void setFocus(boolean hasFocus) {
    if (hasFocus) {
      Color lineColor = UIManager.getColor("Tree.selectionBorderColor");
      setBorder(BorderFactory.createLineBorder(lineColor));
    } else {
      setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
    }    
  }

} 
