/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.util;

import isac.util.formulae.HierarchyKey;
import isac.util.formulae.KEStoreKey;

import java.io.Serializable;

import java.util.Iterator;
import java.util.Vector;

import javax.swing.tree.TreePath;

/**
 * @author Mario Hochreiter, Johannes Loinig
 * @version $Revision$
 *  
 */
public class Hierarchy implements Serializable {

    static final long serialVersionUID = 7791584730774676863L;
    
	/** root node of the whole hierarchy_ */
    protected HierarchyNode root_;

    public Hierarchy() {
    	root_ = new HierarchyNode();
    }

    public void addNode(HierarchyNode parent, HierarchyNode child) {
        if (parent == null) {
            root_ = child;
        } else {
        	parent.add( child );
        }
        
    }

    public Vector<HierarchyNode> getChildren(HierarchyNode parent) {
        return parent.getChildren();
    }

    public HierarchyNode getChild(HierarchyNode parent, int index)
            throws IllegalArgumentException {
    	return parent.getChild( index );
    }

    public int getIndexOfChild(HierarchyNode parent, HierarchyNode child)
            throws IllegalArgumentException {
        Vector<HierarchyNode> children = parent.getChildren();
        if ( children != null ) {
            return children.indexOf(child);
        } else {
            throw new IllegalArgumentException("Parent node has no child!");
        }
    }

    public HierarchyNode getRoot() {
        return root_;
    }
    
    public HierarchyNode getNodeForKEStoreKey(KEStoreKey elem) {
    	return getNodeForKEStoreKey(elem, root_);
    }
    
	/**
	 * @param elem - KEStoreKey of the node we need to find
	 * @param node - the node from where we start the search
	 * @return the node from the specified value, null if node not found
	 */
	public HierarchyNode getNodeForKEStoreKey(KEStoreKey elem, HierarchyNode node ) {
	
		if( elem == null )
			return root_;
		
		if (node.getKEStoreKey() != null && node.getKEStoreKey().equals( elem  ) ) {
			return node;
		}
		System.out.println("elem: "+elem+" kestorekey: "+node.getKEStoreKey());
		
		HierarchyNode result = null;
		Iterator childIterator = node.getChildren().iterator();
		while (childIterator.hasNext()) {
			HierarchyNode currentNode = (HierarchyNode) childIterator.next();
			result = getNodeForKEStoreKey( elem, currentNode);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	/**
	 * @param elem - HierarchyKey.getID of the node we need to find
	 * @return the node from the specified value, null if node not found
	 */	
	public HierarchyNode getNodeForHierarchyKey(Vector<String> elem ) {
		return getNodeForHierarchyKey(elem, root_ );
	}
	
	/**
	 * @param elem - HierarchyKey.getID of the node we need to find
	 * @param node - the node from where we start the search
	 * @return the node from the specified value, null if node not found
	 */
	public HierarchyNode getNodeForHierarchyKey(Vector<String> elem, HierarchyNode node ) {
		Vector<String> id = (Vector<String>) elem.clone();
		
		if( id.size() == 0 )
			return null;
		
		String first_id = id.firstElement();
		
		Vector<HierarchyNode> nodes = node.getChildren();
		Iterator it_nodes = nodes.iterator();
	
		while (it_nodes.hasNext()) {
			HierarchyNode current_node = (HierarchyNode) it_nodes.next();
			
			if( current_node.getId().compareTo( first_id ) == 0 ) {
				if( id.size() == 1 ) {
					return current_node;
				}
				else {
					id.remove( first_id );
					return getNodeForHierarchyKey( id, current_node );
				}
			}
		}
		return null;
	}	
    

    
	/**
	 * To select a single node in a tree we need the wohle path in 
	 * the internal structure. We cant use the hierarchynodes themselfe
	 * to search because we dont know them. So we have to use the only 
	 * thing we know, the HierarchyKey.
	 * 
	 * @param elem is the HierarchyKey.getID of node to select
	 * @return Path to the node with the id or to the root node
	 */
	public TreePath getPath( Vector<String> elem ) {
		HierarchyNode node = getNodeForHierarchyKey( elem, root_ );
		
		if( node == null )
			node = root_;
		
		return getPathToNode(node);
	}
	
	/***
	 * The method returns the TreePath from the root node to the given node
	 */
	public TreePath getPathToNode( HierarchyNode node ) {
		Vector<HierarchyNode> result = new Vector<HierarchyNode>();

		result.add( 0, node );

		while( node != null ) {
			node = node.getParent();
			
			if( node != null ) 
				result.add( 0, node );
		}

		return new TreePath( result.toArray() );
	}
	
	/**
	 * Returns the contentref of a ke store id.
	 * 
	 * @param id
	 * @return
	 */
	public KEStoreKey getKEStoreKey(HierarchyKey id) {
		HierarchyNode node = getNodeForHierarchyKey( id.getID(), root_ );
		
		return node.getKEStoreKey();
	}
	
    private String printNodesHierarchy(HierarchyNode hNode, int level) {
        StringBuffer sb = new StringBuffer();

        if (level == 0) {
            sb.append(hNode.id_).append(" ").append(hNode.ke_store_key_).append(
                    "\n");
        }

        Vector<HierarchyNode> children = this.getChildren(hNode);
        if (children != null) {
            level++;
            Iterator it = children.iterator();
            while (it.hasNext()) {
                HierarchyNode node = (HierarchyNode) it.next();
                StringBuffer levelBuffer = new StringBuffer();
                for (int i = 0; i < level; i++)
                    levelBuffer.append("--");

                sb.append("+").append(levelBuffer).append(": ").append(node.id_)
                        .append(" ").append(node.ke_store_key_).append("\n")
                        .append(printNodesHierarchy(node, level));
            }
        }
        return sb.toString();
    }

    public String toString() {
        return printNodesHierarchy(this.getRoot(), 0);
    }
	
}