/*
 * Created on Sep 11, 2003
 * 
 * Use is subject to license terms.
 */
package isac.util.formulae;

/**
 * Every calculation has a CalcHead, which consists of Model and Specification
 * of the calculation. The CalcHead is empty at the beginning of the
 * Specifying-phase and has to be filled before one can start the solving phase.
 * 
 * @author Alan Krempler
 * @author Manuel Koschuch
 */
public class CalcHead extends CalcElement {

    /**
     * For Serialization purposes
     */
    private static final long serialVersionUID = 1L;

    /**
     * Labels a model item as "correct"
     */
    public static final int MODEL_ITEM_CORRECT = 1;

    /**
     * Labels a model item as "incorrect"
     */
    public static final int MODEL_ITEM_INCORRECT = 2;

    /**
     * Labels a model item as "syntax error"
     */
    public static final int MODEL_ITEM_SYNTAXERROR = 3;

    /**
     * Labels a model item as "incomplete"
     */
    public static final int MODEL_ITEM_INCOMPLETE = 4;

    /**
     * Labels a model item as "false"
     */
    public static final int MODEL_ITEM_FALSE = 5;

    /**
     * Labels a model item as to be filled in by the KI
     */
    public static final int MODEL_ITEM_HELP_ME = 6;

    /**
     * Labels a model item as superfluous
     */
    public static final int MODEL_ITEM_SUPERFLUOUS = 7;

    /**
     * Labels a model item as missing
     */
    public static final int MODEL_ITEM_MISSING = 8;

    /**
     * show all details of a CalcHead in the CalcHeadPanel
     */
    public static final int VIEWSTYLE_FULL = 2;

    /**
     * show the headline and the <code>Model</code> of the
     * <code>CalcHead</code>
     */
    public static final int VIEWSTYLE_MODEL = 3;

    /**
     * show the headline and the <code>Specification</code> of the
     * <code>CalcHead</code>
     */
    public static final int VIEWSTYLE_SPECIFICATION = 4;

    private Position position_;

    private Formula headline_;

    private Model model_;

    private String belongs_to_;

    private Specification specification_;

    /**
     * distinguish between (1) the _overall_-status of a CalcHead and (2) e.g.
     * the MODEL_ITEM_CORRECT for model items as defined above HERE (1) IS MEANT !
     * 
     * can have exactly two values: "correct" xor "incorrect"
     */
    private String status_string_;// TODO.WN041007 confusion with

    // calcHeadStatus

    private int viewstyle_;

    /**
     * Standard Constructor
     */
    public CalcHead() {
        belongs_to_ = "";
        headline_ = new Formula();
        model_ = new Model();
        position_ = new Position();
        specification_ = new Specification();
        status_string_ = "";

    }

    /**
     * Format the object in a way that the bridge can handle it with
     * modifyCalcHead
     * 
     * @return SML representation
     */
    public String toSMLString() {
        StringBuffer sb = new StringBuffer();
        sb.append("(");
        sb.append(this.getPosition().toSMLString());
        sb.append(", ");

        sb.append("\"");
        sb.append(headline_);
        sb.append("\"");
        sb.append(", ");

        sb.append("[");
        sb.append(this.model_.toSMLString());

        sb.append("], " + belongs_to_ + ", (");
        sb.append(specification_.toSMLString());
        sb.append("))");
        return sb.toString();
    }

    /**
     * String representation of CalcHead
     * 
     * @return String representation
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("CalcHead:");
        sb.append(model_.toString());
        sb.append(specification_.toString());
        return sb.toString();
    }

    /**
     * Copies values form one CalcHead object to another. This is needed,
     * because the backend produces a new calc_head_ after each update, but the
     * dialog holds only one instance of CalcHead all the time.
     * 
     * @param other
     *            the other calc_head_ object
     */
    public void fillValuesfrom(CalcHead other) {
        this.belongs_to_ = other.belongs_to_;
        this.headline_ = other.headline_;
        this.position_ = other.position_;

        this.model_ = other.getModel();
        this.specification_ = other.specification_;
        this.visibility_ = other.visibility_;
        this.view_style_ = other.view_style_;
        this.rating_ = other.rating_;
    }

    /**
     * @see isac.util.CalcElement#getMathML()
     * @deprecated because it does NOT make sense for a whole CalcHead
     */
    public String getMathML() {
        return null;
    }

    /**
     * @see isac.util.CalcElement#getType()
     */
    public int getType() {
        return CALCEL_CALCHEAD;
    }

    public Model getModel() {
        return model_;
    }

    public void setModel(Model m) {
        model_ = m;
    }

    public Specification getSpecification() {
        return specification_;
    }

    public void setSpecification(Specification specification) {
        this.specification_ = specification;
    }

    public Position getPosition() {
        return position_;
    }

    public void setPosition(Position position) {
        this.position_ = position;
    }

    // for XMLParserDigest
    public void setResponse(Position position) {
        this.position_ = position;
    }

    public String getBelongsTo() {
        return belongs_to_;
    }

    public void setBelongsTo(String string) {
        belongs_to_ = string;
    }

    public Formula getHeadLine() {
        return headline_;
    }

    /**
     * set the headline in the CalcHead; this is in general e.g.
     * 'Problem(Reals,[by_calculus,maximum])' or for algebrasystem-like problems
     * e.g. 'solve(x+1=2,x)'
     */
    public void setHeadLine(Formula string) {
        headline_ = string;
    }

    /**
     * get the status of the whole CalcHead, which is either "correct" xor
     * "incorrect"
     */
    public String getStatusString() {
        return status_string_;
    }

    public boolean isModelEmpty() {
        if (model_.getNumberOfItems() == 0)
            return true;
        else
            return false;
    }

    /**
     * used by the XMLParserDigester
     * 
     * @see isac.util.parser.XMLParserDigest#parse(String)>
     */
    public void setStatusString(String string) {
        status_string_ = string;
    }

    /**
     * uses this calchead a particular theory (explicitly specified or not) ?
     * 
     * TODO.WN0404 implemented, not included into any detailed design, not used
     */
    public boolean usesTheory(String thyID) {
        return this.specification_.getTheory().equals(thyID);
    }

    /**
     * uses this calchead a particular class of theories (explicitly specified
     * or not) ?
     * 
     * TODO.WN0404 implemented, not included into any detailed design, not used
     */
    public boolean usesSubTheoryOf(String thyID) {
        System.err.println("TODO CalcHead.usesSubTheoryOf(" + thyID + ")");
        System.exit(999);
        return false;
    }

    /**
     * uses this calchead a particular problem (explicitly specified or not) ?
     * 
     * WN0404 implemented, not included into any detailed design, not used
     */
    public boolean usesProblem(ProblemID pid) {
        return this.specification_.getProblem().equals(pid);
    }

    /**
     * uses this calchead a particular class of problems (explicitly specified
     * or not) ?
     * 
     * WN0404 implemented, not included into any detailed design, not used
     */
    public boolean usesProblemClass(ProblemID pid) {
        return ((ProblemID) this.specification_.getProblem())
                .belongsToClass(pid);
    }

    public void setViewstyle(int viewstyle) {
        viewstyle_ = viewstyle;
    }

    public int getViewstyle() {
        return viewstyle_;
    }

    public boolean isComplete() {
        if (status_string_.equals("correct"))
            return true;
        else
            return false;
    }
}