/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.wsdialog;

import isac.interfaces.IToGUI;
import isac.session.SessionManager;
import isac.session.Session;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.UIActionOnWSDialog;
import isac.users.UserLanguage;
import isac.users.UserManager;
import isac.util.Formalization;
import isac.util.ObjectManagerPaths;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;

import isac.util.formulae.KEStoreKey;

import org.apache.log4j.Logger;

/**
 * This class administrates all worksheet dialoges. The worksheet
 * dialog belongs to a Session and contains all opened workseet dialoges
 * of that Session. It also contains the currently active worksheet dialog.
 * Each worksheet gets its own uique ID at construction.
 * 
 * @see isac.wsdialog.WorksheetDialog
 * @see isac.session.Session
 * @author Robert Koenighofer
 */

public class WSDialogManager extends UnicastRemoteObject implements Serializable,
	IWSDialogManager {
	
	static final long serialVersionUID = 348692353;//FIXXME generate with plugin
	
    static Logger logger_;

    private HashMap<String,WorksheetDialog> wsdid_map_wsdialog_;
    
    //FIXXME maybe it is better to store the id only
    private WorksheetDialog active_worksheet_dialog_;

    private int next_wsdid_;
    
    private Session session_;
    
    protected UserLanguage user_language_;

    
    /**
     * Constructor
     * creates a new WSDialogManager
     * 
     * @param session the Session this WSDialogManager belongs to.
     * @throws RemoteException if remote object is not available
     */
    public WSDialogManager(Session session) throws RemoteException{
    	session_ = session;
    	wsdid_map_wsdialog_ = new HashMap<String,WorksheetDialog>();
		logger_ = Logger.getLogger(SessionManager.class.getName());
		active_worksheet_dialog_ = null;
		next_wsdid_ = 0;
		
		user_language_ = session_.getLanguage();
    }
    
    /**
     * This method creates a new WorksheetDialog and returns
     * the ID of the created Object. The new Dialog is added into
     * the map and is set as active Worksheet dialog.
     * 
     * @return the ID of the created Worksheet dialog.
     * @throws RemoteException if remote object is not available
     */
    public String openWSDialog(Formalization f, KEStoreKey example) throws RemoteException {
        String wsd_id = session_.getId() + "_" + next_wsdid_++;
        logger_.fatal(" SM->DG: newWorksheetDialog(localhost, isac/xmldata/users/"
                        + session_.getId() + ")");
        
        WorksheetDialog new_dialog = new WorksheetDialog(
        		ObjectManagerPaths.MATH_ENGINE_PATH, 
        		session_,
                wsd_id,
                f);
    	
        wsdid_map_wsdialog_.put(wsd_id,new_dialog);
        
        if( example != null )
        	session_.addWorksheetWithExample(wsd_id, example);
    	
        active_worksheet_dialog_ = new_dialog;
        
        IToGUI ifc_to_gui = session_.getIToGUI();
        
        UIActionOnWSDialog uaowsd = new UIActionOnWSDialog( EUIElement.UI_OPEN_WORKSHEET,
        		EUIContext.UI_CONTEXT_WINDOW_APPLICATION, user_language_,
        		new_dialog );
        
        ifc_to_gui.doUIAction( uaowsd );
        
        return wsd_id;
    }
    
    /**
     * This method creates a new WorksheetDialog out of an example
     * and returns the ID of the created Object. The new Dialog is added into
     * the map and is set as active Worksheet dialog. The HierarchyKey of
     * the example is stored together with the worksheet dialog id in
     * a map in the session.
     * 
     * @param 
     * @return the ID of the created Worksheet dialog.
     * @throws RemoteException if remote object is not available
     */
    public String openWSDialog() throws RemoteException {
    	String new_dialog_id = openWSDialog( new Formalization(), null );
        return new_dialog_id;
    }
    
    /**
     * This method returns the WorksheetDialog with the desired
     * ID. If the ID is unknown to the WSDialogManager, null is
     * returned.
     * 
     * @return the WorksheetDialog with the desired ID or null if it does not exist.
     * @throws RemoteException if remote object is not available
     */
    public WorksheetDialog getWSDialog(String wsd_id) throws RemoteException {
    	return wsdid_map_wsdialog_.get(wsd_id);
    }
    
    /**
     * This method closes the WorksheetDialog with the specified ID. If
     * the ID is unknown to the WSDialogManager nothing will happen.
     * If the worksheet to close is the active worksheet, the worksheet
     * with the highest ID will become the new active worksheet. If there
     * is no worksheet left, the active worksheet will be set to null.
     * 
     * @param wsd_id the ID of the worksheet to close 
     * @throws RemoteException if remote object is not available
     */
    public void closeWSDialog(String wsd_id) throws RemoteException{
    	WorksheetDialog dialog_to_remove = wsdid_map_wsdialog_.get(wsd_id);
    	wsdid_map_wsdialog_.remove(wsd_id);
    	session_.removeWorksheetWithExample(wsd_id);
    	if(dialog_to_remove != active_worksheet_dialog_)
    		return;
    	Object[] dialog_array = wsdid_map_wsdialog_.values().toArray();
    	if(dialog_array.length == 0) {
    		active_worksheet_dialog_ = null;
    		return;
    	}    		
    	active_worksheet_dialog_ = (WorksheetDialog) dialog_array[dialog_array.length - 1];
    }
    
    /**
     * This method returns the active WorksheetDialog. If there
     * is no active WorksheetDialog (e.g. if there is no WorksheetDialog
     * at all), null is returned.
     * 
     * @return The active WorksheetDialog or null if there is non.
     * @throws RemoteException if remote object is not available
     */
    public WorksheetDialog getActiveWSDialog() throws RemoteException {
    	return active_worksheet_dialog_;
    }
    
    /**
     * This method sets the WorksheetDialog with the specified
     * ID as active WorksheetDialog. If no WorksheetDialog with this
     * ID is known, the active worksheet dialoge remains unchanged.
     * If the passed String is null, the active WorksheetDialog is
     * set to null, which means, that there is no active Worksheet at
     * the moment.
     * 
     * @param wsd_id the ID of the worksheet to set active
     * @throws RemoteException if remote object is not available
     */
    public void setActiveWSDialog(String wsd_id) throws RemoteException{
    	active_worksheet_dialog_ = wsdid_map_wsdialog_.get(wsd_id);
    }
    
    /**
     * This method returns the number of currently opened
     * worksheet dialoges.
     * 
     * @return the number of currently opened worksheets. 
     * @throws RemoteException if remote object is not available
     */
    public int numberOpenedWSDialoges() throws RemoteException{
    	return wsdid_map_wsdialog_.size();
    }
}
