package isac.wsdialog;

import isac.bridge.*;
import isac.browserdialog.ExampleDialog;
import isac.browserdialog.IContextPresenter;
import isac.browserdialog.MethodDialog;
import isac.browserdialog.ProblemDialog;
import isac.browserdialog.TheoryDialog;
import isac.interfaces.*;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionOnCalcElement;
import isac.useractions.UserActionOnCalcElement;
import isac.useractions.UserActionOnPosition;
import isac.users.IUserSettings;
import isac.users.User;
import isac.users.UserLanguage;
import isac.users.UserSettings;
import isac.util.*;
import isac.util.formulae.*;
import isac.util.tactics.*;
import isac.session.Session;
import isac.session.SessionManager;
import isac.session.ISessionManager;
import isac.users.UserManager;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.rmi.*;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * The WorksheetDialog moderates the communication between two instances working
 * on the same CalcTree object. One of these instances is the user, the other a
 * math engine. Most probably, the user's GUI, the WorksheetDialog and the math
 * engine reside on different machines and communicate with each other by means
 * of RMI.
 * 
 * @author Alan Krempler
 * @author Manuel Koschuch
 */
public class WorksheetDialog extends UnicastRemoteObject implements
		IWorksheetDialog, IToUser, Serializable,IContextProvider {

	static final long serialVersionUID = 4952921581668118572L;
	
	static Logger logger_ = Logger.getLogger(BridgeRMI.class.getName());

	protected MathEngine math_engine_;

	protected CalcTree calc_tree_;

	/**
	 * The active formula in the mathematic engine
	 */
	protected ICalcIterator active_formula_;
	
	/**
	 * Represents the formula on which the context depends.
	 * It is the last selected formula in the worksheet or
	 * the active formula from the mathematic engine 
	 * 
	 */
	protected ICalcIterator context_formula_;
	
	protected Session session_;

	/**
	 * determines if the calculation on the worksheet is ready
	 */
	protected boolean worksheet_ready_;

	protected int phase_;

	private Vector<IToUser> datachange_listeners_;

	private IToGUI ui_control_listener_;

	protected String worksheet_id_;

	protected UserSettings user_settings_;// hold whole hashmap locally

	protected ISessionManager session_manager_;

	protected UserLanguage user_language_;
	
	/**
	 * @param ME_path
	 *            URL of the math engine to be used
	 * @param user_path
	 *            path(without URL) to UserSettings, UserModel
	 * @param session_dialog
	 * @param session_id
	 *            which create the WorksheetDialog and Worksheet
	 * @throws RemoteException
	 */
	public WorksheetDialog(String ME_path, Session session, String worksheet_id, Formalization f)
			throws RemoteException {
		initialise( ME_path, session, worksheet_id, f );
	}
	
	public WorksheetDialog(String ME_path, Session session, String worksheet_id)
	throws RemoteException {
		initialise( ME_path, session, worksheet_id, new Formalization() );
	}
	
	void initialise( String ME_path, Session session, String worksheet_id, Formalization f ) throws RemoteException {
		this.rmiBind();
		datachange_listeners_ = new Vector<IToUser>();
		
		MathEngine.init(ME_path);
		math_engine_ = MathEngine.getMathEngine();
		
		phase_ = DIALOGPHASE_IDLE;

		session_ = session;
		
		User user = null;
		
		user = UserManager.getInstance().getUser(session.getId());
	
		user_settings_ = user.getUserSettings();
		user_language_ = user.getLanguage();
		
		worksheet_id_ = worksheet_id;

		session_manager_ = SessionManager.getInstance();	
		
		getCalcTree( f );
	}

	public String getWorksheetID() {
		return worksheet_id_;
	}

	public CalcTree getCalcTree() {
		return calc_tree_;
	}
	
	/**
	 * start a calculation
	 */
	public void getCalcTree( Formalization f ) {
		// do administrative stuff
		calc_tree_ = math_engine_.getCalcTree(f);
		calc_tree_.addDataChangeListener(this);

		switch (user_settings_.getValue(IUserSettings.KEY1_START_SPECIFY)) {
		case IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE:
			phase_ = DIALOGPHASE_SOLVE;
			break;
		case IUserSettings.VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS:
			phase_ = DIALOGPHASE_SPECIFY;
			break;
		}

		return;
	}

	/**
	 * enter the specifying-phase; 2 cases: (1) at the beginning of a
	 * calculation as well as of a subproblem; sets the Descriptions in the
	 * Model (to support user-input) and in CAS-commands completes the Model.
	 * (2) for inspection of a "correct" CalcHead somewhere above the
	 * activeFormula; in this case the CalcHead is not updated, it is just the
	 * starting point into the math-knowledge.
	 * 
	 * @return CalcHead eg. for providing CalcHead#VIEWSTYLE_*
	 * 
	 * @see isac.wsdialog.IWorksheetDialog#startSpecifying()
	 */
	private void startSpecifying(CalcHead calc_head) {
		phase_ = DIALOGPHASE_SPECIFY;
		try {
			// remove solving buttons
			removeNextAndAutoButtonForWorksheet();
			if (calc_head.getStatusString().equals("incorrect")) {
				// case (1)
				// -----------------------------------------------------------
				// if you think, modelProblem should only set the descriptions
				// in the Model (and NOT complete it for CAS-commands)
				// search the sml-code for WN060225-modelProblem
				calc_head = calc_tree_.modelProblem();
				// -----------------------------------------------------------
				doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_OPEN_CALCHEAD,
						EUIContext.UI_CONTEXT_SWITCH_PHASES, user_language_, calc_head));
				addCalcHeadPanelButtons();
				addNextAndAutoButtonsForCalchead();

			} else {// case (2)
				// only Button Close
				doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_OPEN_CALCHEAD,
						EUIContext.UI_CONTEXT_SWITCH_PHASES, user_language_, calc_head));
				ui_control_listener_.addUIElement(new UIAction(
						EUIElement.UI_SPECIFY_CLOSE_CALCHEAD,
						EUIContext.UI_CONTEXT_CALCHEAD, user_language_));
				addNextAndAutoButtonForWorksheet();
			}

		} catch (RemoteException e) {
			e.printStackTrace();
		}

	}

	/**
	 * @see isac.wsdialog.IWorksheetDialog#startCalculate()
	 */
	public void startCalculate() {
		calc_tree_.startCalculate();
	}

	/**
	 * dismiss the CalcHeadPanel and start WorksheetDialog#DIALOGPHASE_SOLVE
	 * 
	 * @see isac.interfaces.IToCalc#startSolving()
	 */
	private void startSolving() {
		try {
			removeCalcHeadPanelButtons();
			removeNextAndAutoButtonsForCalchead();
			addNextAndAutoButtonForWorksheet();

		} catch (RemoteException e) {
			e.printStackTrace();
		}
		phase_ = DIALOGPHASE_SOLVE;
	}

	private void buildUIElementsContextCalcElements() {
		try {
			// ui_control_listener_.addUIElement(new UIAction(
			// UI_SOLVE_GET_APPLICABLE_TACTICS,
			// EUIContext.UI_CONTEXT_CALCELEMENTS));
			ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SOLVE_SHOW_ASSUMPTIONS,
					EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
			ui_control_listener_
					.addUIElement(new UIAction(EUIElement.UI_SOLVE_TACTIC_APPLIED,
							EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
			ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SOLVE_SHOW_INTERMEDIATE_STEPS,
					EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_));
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	private void addNextAndAutoButtonForWorksheet() throws RemoteException {
		// Button next
		ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SOLVE_CALCULATE_1,
				EUIContext.UI_CONTEXT_CALCULATION, user_language_));
		// Button auto
		ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SOLVE_CALCULATE_ALL,
				EUIContext.UI_CONTEXT_CALCULATION, user_language_));
	}

	private void removeNextAndAutoButtonForWorksheet() throws RemoteException {
		// Button next
		ui_control_listener_.removeUIElement(new UIAction(EUIElement.UI_SOLVE_CALCULATE_1,
				EUIContext.UI_CONTEXT_CALCULATION, user_language_));
		// Button auto
		ui_control_listener_.removeUIElement(new UIAction(EUIElement.UI_SOLVE_CALCULATE_ALL, EUIContext.UI_CONTEXT_CALCULATION, user_language_));

	}

	private void addCalcHeadPanelButtons() throws RemoteException {
		// Button Complete
		ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SPECIFY_COMPLETE_CALCHEAD, EUIContext.UI_CONTEXT_CALCHEAD, user_language_));
		// // Button Check
		// ui_control_listener_.addUIElement(new UIAction(
		// UI_SPECIFY_CHECK_CALCHEAD, EUIContext.UI_CONTEXT_CALCHEAD));
		// Button reset
		ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SPECIFY_RESET,
				EUIContext.UI_CONTEXT_CALCHEAD, user_language_));
		// Button Start Solving
		ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SPECIFY_TO_SOLVE,
				EUIContext.UI_CONTEXT_CALCHEAD, user_language_));

	}

	private void removeCalcHeadPanelButtons() throws RemoteException {
		// Button Complete
		ui_control_listener_.removeUIElement(new UIAction(EUIElement.UI_SPECIFY_COMPLETE_CALCHEAD, EUIContext.UI_CONTEXT_CALCHEAD,user_language_));
		// Button Start Solving
		ui_control_listener_.removeUIElement(new UIAction(EUIElement.UI_SPECIFY_TO_SOLVE,
				EUIContext.UI_CONTEXT_CALCHEAD,user_language_));
		// // Button Check
		// ui_control_listener_.removeUIElement(new UIAction(
		// UI_SPECIFY_CHECK_CALCHEAD, EUIContext.UI_CONTEXT_CALCHEAD));
		// Button reset
		ui_control_listener_.removeUIElement(new UIAction(EUIElement.UI_SPECIFY_RESET,
				EUIContext.UI_CONTEXT_CALCHEAD,user_language_));
	}

	/**
	 * add next and auto Button to WindowApplication
	 */
	// Context Calculation is used instead of context calchead,
	// because buttons have to be added to windowapplication and not to
	// calcheadpanel
	private void addNextAndAutoButtonsForCalchead() {
		try {
			// add next Button
			ui_control_listener_.addUIElement(new UIAction(EUIElement.UI_SPECIFY_COMPLETE_STEPWISE,
					EUIContext.UI_CONTEXT_CALCULATION, user_language_ ) );
			// add auto Button
			ui_control_listener_
					.addUIElement(new UIAction(EUIElement.UI_SPECIFY_CALCULATE_ALL,
							EUIContext.UI_CONTEXT_CALCULATION, user_language_ ) );
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	private void removeNextAndAutoButtonsForCalchead() {

		// Context Calculation is used instead of context calchead,
		// because buttons have to be added to windowapplication and not to
		// calcheadpanel
		try {
			// add next Button
			ui_control_listener_.removeUIElement(new UIAction(EUIElement.UI_SPECIFY_COMPLETE_STEPWISE,
					EUIContext.UI_CONTEXT_CALCULATION, user_language_) );
			// add auto Button
			ui_control_listener_
					.removeUIElement(new UIAction(EUIElement.UI_SPECIFY_CALCULATE_ALL,
							EUIContext.UI_CONTEXT_CALCULATION, user_language_ ) );
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * accept the changes in a calculation reported by the bridge
	 * 
	 * ... and pass them to the Worksheet
	 * 
	 * @see isac.wsdialog.IToUser#calcChanged(isac.wsdialog.CalcChangedEvent)
	 */
	public void calcChanged(CalcChangedEvent event) throws RemoteException {
		worksheet_ready_ = false;
		logger_.fatal(" WS<-. . : calcChanged (unc="
				+ event.getLastUnchangedFormula().toSMLString() + ", del="
				+ event.getLastDeletedFormula().toSMLString() + ", gen="
				+ event.getLastGeneratedFormula().toSMLString());
		try {
			for (int i = 0; i < datachange_listeners_.size(); i++) {
				CalcChangedEvent wrappedEvent = new CalcChangedEvent(event
						.getTransactionId(), event.isCompleted(),
						new DialogIterator((CalcIterator) event
								.getLastUnchangedFormula(), this),
						new DialogIterator((CalcIterator) event
								.getLastDeletedFormula(), this),
						new DialogIterator((CalcIterator) event
								.getLastGeneratedFormula(), this));
				((IToUser) datachange_listeners_.elementAt(i))
						.calcChanged(wrappedEvent);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		// WN060202 this does NOT work with MORE than 1 worksheet...
		// if (event.getLastGeneratedFormula().onEndOfCalculation())
		// removeNextAndAutoButtonForWorksheet();
		// ...different calls in different phases???
		
		addUIElementsContextOneElement(event);
		
		active_formula_ = event.getLastGeneratedFormula();
		
		context_formula_ = (ICalcIterator) event.getLastGeneratedFormula().clone();
		
		updateContextFormulaOnWorksheet( context_formula_ );
		
		notifyContextPresenter();
		
		if (active_formula_.onCalcHead()) {
			switch (user_settings_.getValue("KEY1_START_SPECIFY")) {
			case IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE:
				// WN050728
				// instead calc_tree_.autoCalculate(IToCalc.SCOPE_CALCULATION,
				// 1);
				// ...NEW Model_Problem (inserting Descriptions into the Model)
				// without causing CalcChangedEvent (would loop in calcChanged!)
				startSolving();
				break;
			case IUserSettings.VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS:
				startSpecifying((CalcHead) active_formula_.getElement());
				break;
			default:
				try {
					throw new Exception("WorksheetDialog#calcChanged:switch"
							+ " (user_settings_.getValue(KEY1_START_SPECIFY)");
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		}
	}

	/**
	 * construct the context-menu on one specific calcelement (e.g. with
	 * 'Calchead' and without)
	 * 
	 */
	private void addUIElementsContextOneElement(CalcChangedEvent event) {
		ICalcIterator from = event.getLastUnchangedFormula();
		ICalcIterator to   = event.getLastGeneratedFormula();
		
		Vector calc_formulae = null;

		try {
			calc_formulae = from.getFormulaeFromTo( to, new Integer(0), false  );
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}

		if( calc_formulae == null )
			return;
		
    	Iterator vector_it = calc_formulae.iterator();
    	
    	while( vector_it.hasNext() ) {
    		CalcFormula cf = (CalcFormula) vector_it.next();
    		
    		if( cf.getPosition().getKind().equals("Pbl") 
    				||	cf.getPosition().getKind().equals("Met") )	{
    			try {
					ui_control_listener_.addUIElement(new UIAction(
							EUIElement.UI_SOLVE_SHOW_CALCHEAD,
							EUIContext.UI_CONTEXT_ONEELEMENT, 
							cf.getPosition(), user_language_ 
							)
						);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    	} 
	}

	/***
	 * This method sends the applicable tactics to the specific item
	 * in the worksheet determined by the Position of the iterator
	 * 
	 * @param it
	 */
	private void sendApplicableTacticsToWorksheet(ICalcIterator it) {
		try {
			Vector tactics = it.getApplicableTactics(ICalcIterator.TACTICS_ALL);

			if (tactics != null) {
				for (int i = 0; i < tactics.size(); i++)
					ui_control_listener_
							.addUIElement(new UIActionOnCalcElement(EUIElement.UI_SOLVE_SET_NEXT_TACTIC,
									EUIContext.UI_CONTEXT_APPLICABLE_TACTIC, user_language_, it
											.getPosition(), (Tactic) tactics
											.get(i)));
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
	
	private void updateContextFormulaOnWorksheet(ICalcIterator it) {
		try {
			ui_control_listener_.doUIAction(
					new UIAction(
							EUIElement.UI_SOLVE_CHANGE_CONTEXT_FORMULA,
							EUIContext.UI_CONTEXT_ONEELEMENT,   
							it.getPosition(), user_language_
							) 
					);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see isac.wsdialog.IWorksheetDialog#iterator(CalcIterator,
	 *      IWorksheetDialog)
	 * @see isac.interfaces.IToCalc#iterator()
	 */
	public ICalcIterator iterator(CalcIterator it, IWorksheetDialog dg) {
		try {
			return new DialogIterator(it, dg);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * @see isac.wsdialog.IToCalc#addListener(isac.wsdialog.IToUser)
	 */
	public boolean addDataChangeListener(IToUser listener)
			throws RemoteException {
		if (datachange_listeners_.contains(listener)) {
			return false;
		} else {
			datachange_listeners_.add(listener);
		}
		return true;
	}

	public boolean registerUIControlListener(IToGUI listener)
			throws RemoteException {
		ui_control_listener_ = listener;
		buildUIElementsContextCalcElements();
		return true;
	}

	/**
	 * Notify the Dialog Guide about interaction from the User DO NOT MODIFY THE
	 * SEQUENCE OF THE CASE BLOCKS IN THE SWITCH STATEMENTS!!
	 * 
	 * @param action
	 * @return true, if the action has been processed, false if the processing
	 *         the action has been denied
	 * @throws RemoteException
	 * 
	 */
	public boolean notifyUserAction(IUserAction action)
			throws DialogProtocolException, RemoteException {
		EUIElement request = action.getActionID();
		CalcHead calc_head = new CalcHead();
		CalcHead ch_to_check = new CalcHead();

		logger_.fatal(" WS->DG: notifyUserAction request= " + request);
		System.out.println(" WS->DG: notifyUserAction request= " + request);

		
		/// the requests that do not depend on the phase
		switch( request ) {
		case UI_CLOSE_WORKSHEET:
			calc_tree_.destruct();
			session_.getWSDialogManager().closeWSDialog(worksheet_id_);
			if( session_.getWSDialogManager().numberOpenedWSDialoges() == 0 )
				removeNextAndAutoButtonForWorksheet();
			IContextPresenter bdg = session_.getContextPresenter();
			ContextUnknown context_unknown = new ContextUnknown();			
			bdg.presentContext( context_unknown );
			return true;
		case UI_ACTIVE_WORKSHEET:
			session_.getWSDialogManager().setActiveWSDialog( worksheet_id_ );
			notifyContextPresenter();
			return true;
		}

		switch (phase_) {
		case DIALOGPHASE_IDLE:
			throw new DialogProtocolException(request, phase_);
		case DIALOGPHASE_SPECIFY:
			if (request.ordinal() < EUIElement.UI_DUMMY_FIRST_SPECIFY.ordinal()
					|| request.ordinal() > EUIElement.UI_DUMMY_LAST_SPECIFY.ordinal())
				throw new DialogProtocolException(request, phase_);
			break;
		case DIALOGPHASE_SOLVE:
			if (request.ordinal() < EUIElement.UI_DUMMY_FIRST_SOLVE.ordinal() 
					|| request.ordinal() > EUIElement.UI_DUMMY_LAST_SOLVE.ordinal() )
				throw new DialogProtocolException(request, phase_);
			break;
		}

		// *********** handle IUserAction's **********************

		switch (request) {
		// in solve-phase request open a CalcHead by context-menu
		case UI_SOLVE_SHOW_CALCHEAD:
			calc_head = (CalcHead) context_formula_.getElement();
			// calc_head = (CalcHead) ((UserActionOnIterator) action)
			// .getPosition().getElement();
			startSpecifying(calc_head);
			//FIXXME RK 060712 here was: problem_browser_dialog_.setContext(worksheet_id_);
			break;

		// Enter pressed after typing formula
		case UI_SPECIFY_EDIT_ACTIVE_FORMULA_COMPLETE:
			ch_to_check = (CalcHead) ((UserActionOnCalcElement) action)
					.getCalcElement();
			calc_head = calc_tree_.checkCalcHead(ch_to_check);
			doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_CHECK_CALCHEAD,
					EUIContext.UI_CONTEXT_CALCHEAD, user_language_, calc_head));
			break;

		// Complete pressed
		case UI_SPECIFY_COMPLETE_CALCHEAD:
			if (!calc_head.isComplete()) {
				calc_head = calc_tree_.completeCalcHead();
				doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_COMPLETE_CALCHEAD,
						EUIContext.UI_CONTEXT_CALCHEAD, user_language_, calc_head));
			}
			break;
		// Radiobutton for Method/Problem clicked
//		case UI_SPECIFY_TOGGLE_METHOD_PROBLEM:
//			System.out.println("toggle");
//			// WN050830 use UI_SPECIFY_SWITCH_TO_GUARD and
//			// UI_SPECIFY_SWITCH_TO_MODEL instead,
//			// and use some of this code ...
//			// ICalcIterator ci = calc_tree_.getActiveFormula();
//			// ci.switchToGuard();
//			// CalcHead ch = (CalcHead) ci.getElement();
//			//
//			// TODO MK add code here to get new CalcHead with either Met or Pbl
//			// Elements
//
//			// ch_to_check = (CalcHead) ((UserActionOnCalcElement) action)
//			// .getCalcElement();
//			// calc_head = calc_tree_.checkCalcHead(ch_to_check);
//			// doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_CHECK_CALCHEAD,
//			// EUIContext.UI_CONTEXT_CALCHEAD, user_language_, user_language_, calc_head));
//			break;

		case UI_SPECIFY_METHOD:
		case UI_SPECIFY_THEORY:
		case UI_SPECIFY_PROBLEM:
		case UI_SPECIFY_COMPLETE_GIVEN:
		case UI_SPECIFY_COMPLETE_FIND:
		case UI_SPECIFY_COMPLETE_RELATE:
			throw new DialogNotImplementedException(request, phase_);

		// check pressed
		case UI_SPECIFY_CHECK_CALCHEAD:
			ch_to_check = (CalcHead) ((UserActionOnCalcElement) action)
					.getCalcElement();
			calc_head = calc_tree_.checkCalcHead(ch_to_check);
			doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_CHECK_CALCHEAD,
					EUIContext.UI_CONTEXT_CALCHEAD, user_language_, calc_head));
			break;

		// next pressed
		case UI_SPECIFY_COMPLETE_STEPWISE:
			if (!calc_head.isComplete()) {
				calc_head = calc_tree_.nextSpecify();
				doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_COMPLETE_STEPWISE,
						EUIContext.UI_CONTEXT_CALCHEAD, user_language_, calc_head));
			}
			break;

		// close pressed
		case UI_SPECIFY_CLOSE_CALCHEAD:
			// this action is enabled _ONLY_ if the Panel was opened read-only
			// on a complete CalcHead just for inspection
			doUIAction(new UIAction(EUIElement.UI_SPECIFY_CLOSE_CALCHEAD,
					EUIContext.UI_CONTEXT_SWITCH_PHASES, user_language_ ));
			phase_ = DIALOGPHASE_SOLVE;
			//FIXXME RK 060712 here was: problem_browser_dialog_.setContext("");
			break;

		// reset pressed
		case UI_SPECIFY_RESET:
			calc_tree_.resetCalcHead();
			calc_head = calc_tree_.modelProblem();
			doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_RESET,
					EUIContext.UI_CONTEXT_CALCHEAD, user_language_, calc_head));
			break;

		// start solving pressed
		case UI_SPECIFY_TO_SOLVE:
			//FIXXME RK 060712 here was: problem_browser_dialog_.setContext("");
			
		// ch_to_check = (CalcHead) ((UserActionOnCalcElement) action)
		// .getCalcElement();
		// calc_head = calc_tree_.checkCalcHead(ch_to_check);
		// if (!calc_head.isComplete()) {
		// calc_head = calc_tree_.completeCalcHead();
		// doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_CHECK_CALCHEAD,
		// EUIContext.UI_CONTEXT_CALCHEAD, user_language_, user_language_, calc_head));
		// break;
		// }

		// auto pressed
		case UI_SPECIFY_CALCULATE_ALL:
			if (!calc_head.isComplete()) {
				calc_head = calc_tree_.completeCalcHead();
				if (!calc_head.isComplete()) {
					calc_tree_.resetCalcHead();
					calc_head = calc_tree_.completeCalcHead();
				}
			}

			doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_CALCULATE_ALL,
					EUIContext.UI_CONTEXT_CALCHEAD, user_language_, calc_head));
			doUIAction(new UIAction(EUIElement.UI_SPECIFY_CLOSE_CALCHEAD,
					EUIContext.UI_CONTEXT_SWITCH_PHASES, user_language_ ));
			startSolving();

		// --------------solving phase------------------------------------
		case UI_SOLVE_CALCULATE_1:
			// handle the CalcHead such that its HeadLine can be
			// presented like a CalcFormula
			if (active_formula_.onCalcHead()) {
				calc_tree_.completeCalcHead();
				try {
					calc_tree_.startSolving();
				} catch (AutoCalculateException e1) {
					e1.printStackTrace();
				}
			} else {
				calc_tree_.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
			}
			break;
			
		case UI_SOLVE_CALCULATE_ALL:
			switch (request) {
			case UI_SPECIFY_CALCULATE_ALL:
			case UI_SOLVE_CALCULATE_ALL:
				calc_tree_.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
				break;
			case UI_SPECIFY_TO_SOLVE:
			default:
			}
			break;

		case UI_SOLVE_CALCULATE_SUBPROBLEM:
			calc_tree_.autoCalculate(IToCalc.SCOPE_SUBPROBLEM, 0);
			break;

		case UI_SOLVE_EDIT_ACTIVE_FORMULA:
			ui_control_listener_.doUIAction(new UIAction(EUIElement.UI_DO_EDIT_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, user_language_ ));
			break;

		case UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE:
			System.out.println("**********XXX****************");
			CalcFormula formula = (CalcFormula) ((UserActionOnCalcElement) action).getCalcElement();
			// if the position is null, the formula did not exist, so we try to
			// append a new formula
			if (formula.getPosition() == null) {
				calc_tree_.appendFormula(formula);
			} else {
				active_formula_ = new CalcIterator(calc_tree_, formula
						.getPosition());
				calc_tree_.replaceFormula(formula);
			}
			break;

		case UI_SOLVE_APPEND_USER_FORMULA:
			ui_control_listener_.doUIAction(new UIAction(EUIElement.UI_DO_APPEND_FORMULA, EUIContext.UI_CONTEXT_ONEELEMENT, user_language_ ));
			break;

		case UI_SOLVE_MOVE_ACTIVE_FORMULA: {
			UserActionOnPosition uaop 	= (UserActionOnPosition) action;
			Position activePosition 	= uaop.getPosition();
			ICalcIterator iterator 		= new CalcIterator( calc_tree_, activePosition );
			calc_tree_.moveActiveFormula(iterator);
		}
		break;
			
		case UI_SOLVE_CHANGE_CONTEXT_FORMULA: {
			UserActionOnPosition uaop 	= (UserActionOnPosition) action;
			Position activePosition 	= uaop.getPosition();
			if( activePosition != null ) {
				context_formula_ = new CalcIterator( calc_tree_, activePosition );	
				notifyContextPresenter();
			}
		}
		break;

		case UI_SOLVE_GET_PROPOSED_TACTIC:
			throw new DialogNotImplementedException(request, phase_);

		case UI_SOLVE_GET_APPLICABLE_TACTICS:
			sendApplicableTacticsToWorksheet( context_formula_ );
			doUIAction(new UIAction(EUIElement.UI_SOLVE_SHOW_APPLICABLE_TACTICS,
					EUIContext.UI_CONTEXT_ONEELEMENT, user_language_ ));
			break;

		case UI_SOLVE_SET_NEXT_TACTIC:
			UserActionOnCalcElement uaoce = (UserActionOnCalcElement) action;

			calc_tree_.moveActiveFormula(new CalcIterator(calc_tree_, uaoce.getPosition()));
			calc_tree_.setNextTactic((Tactic) uaoce.getCalcElement());
			calc_tree_.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
			break;

		case UI_SOLVE_HELP_ENTERING_FORMULA:
			throw new DialogNotImplementedException(request, phase_);

		case UI_SOLVE_SHOW_ASSUMPTIONS: {
			Assumptions asm = context_formula_.getAssumptions();
			System.out.println("in WSDialog: asm = " + asm.toSMLString());
			doUIAction(new UIActionOnCalcElement(EUIElement.UI_SOLVE_SHOW_ASSUMPTIONS,
					EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_, asm) );
		}
		break;

		case UI_SOLVE_SHOW_INTERMEDIATE_STEPS: {
			calc_tree_.intermediateSteps(context_formula_); 
		}
		break;

		case UI_SOLVE_TACTIC_APPLIED: {
			Tactic ta = context_formula_.getTactic();
			doUIAction(new UIActionOnCalcElement(EUIElement.UI_SOLVE_TACTIC_APPLIED,
					EUIContext.UI_CONTEXT_CALCELEMENTS, user_language_, ta));
			break;
		}

		case UI_SPECIFY_OPEN_CALCHEAD:
			System.out.println("Pressed!");
			break;

		default:
			throw new DialogUnknownActionException(request, phase_);
		}
		return true;
	}

	private void rmiBind() {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

		try {
			LocateRegistry
					.createRegistry(ObjectManagerPaths.OBJECT_MANAGER_PORT);
		} catch (java.rmi.RemoteException exc2) {
			System.err.println("can not create registry: " + exc2.getMessage());
		}

		// fixed path name =
		// //localhost/isac-WorksheetDialog...Naming.rebind(name, this);
		String name = ObjectManagerPaths.WORKSHEET_DIALOG_PATH; // "//localhost/isac-WorksheetDialog";
		try {
			System.out.println("try to bind as " + name);
			Naming.rebind(name, this);
			System.out.println("Dialog Guide bound to " + name);
			// WN040906 was Object Manager ...
		} catch (java.rmi.ConnectException e) {
			System.err.println("failed to contact as " + name
					+ " (creating RMI-Server on localhost: 1099)");
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see isac.wsdialog.IWorksheetDialog#notifyGUIReady()
	 */
	public void notifyGUIReady() throws RemoteException {
		worksheet_ready_ = true;
	}

	/**
	 * @see isac.interfaces.IToGUI#doUIAction(isac.wsdialog.EUIContext)
	 */
	public synchronized void doUIAction(final IUIAction action)
			throws RemoteException {
		Thread t = new Thread(new Runnable() {

			public void run() {
				try {
					while (worksheet_ready_ == false) {
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e1) {
							e1.printStackTrace();
						}
					}
					ui_control_listener_.doUIAction(action);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
		});
		t.run();
	}

	/**
	 * request the gui to add an interactive element
	 * 
	 * @see isac.interfaces.IToGUI#addUIElement(isac.wsdialog.IUIAction)
	 */
	public synchronized void addUIElement(final IUIAction action)
			throws RemoteException {
		Thread t = new Thread(new Runnable() {

			public void run() {
				try {
					while (worksheet_ready_ == false) {
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e1) {
							e1.printStackTrace();
						}
					}
					ui_control_listener_.addUIElement(action);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
		});
		t.run();
	}

	/**
	 * request the gui to remove an interactive element
	 * 
	 * @see isac.interfaces.IToGUI#removeUIElement(isac.wsdialog.IUIAction)
	 */
	public synchronized void removeUIElement(final IUIAction action)
			throws RemoteException {
		Thread t = new Thread(new Runnable() {

			public void run() {
				try {
					while (worksheet_ready_ == false) {
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e1) {
							e1.printStackTrace();
						}
					}
					ui_control_listener_.removeUIElement(action);
					// ???worksheet_ready_ = false;
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
		});
		t.run();
	}

	
	/***
	 *  TODO: notify the active context presenter that something happened
	 */
	private void notifyContextPresenter() {
		System.out.println( "In WorksheetDialog::notifyContextPresenter !!" );
		
		Context current_context = new ContextUnknown();
		IContextPresenter bdg = session_.getContextPresenter();
		
		if( bdg == null ) {
			System.out.println("No dialog");
			return;
		}
		
		if( bdg instanceof ExampleDialog ) {
			System.out.println("Example dialog");
			current_context = initContext( ContextType.CONTEXT_EXAMPLES );
		} else if( bdg instanceof ProblemDialog ) {
			System.out.println("Problem dialog");
			current_context = initContext( ContextType.CONTEXT_PROBLEMS );
		} else if( bdg instanceof MethodDialog ) {
			System.out.println("Method dialog");
			current_context = initContext( ContextType.CONTEXT_METHODS );
		} else if( bdg instanceof TheoryDialog ) {
			System.out.println("Theory dialog");
			current_context = initContext( ContextType.CONTEXT_THEORIES );
		}
		
		bdg.presentContext( current_context );
	}
	
	/**
	 * @see isac.wsdialog.IContextProvider#initContext(ContextMethod)
	 * 
	 * @throws RemoteException
	 * 
	 */
	public Context initContext( ContextType type ) {
		System.out.println("In WorksheetDialog::initContext() : type is " + type );
		
		Context current_context = new ContextUnknown();
		
		switch( type ) {
		case CONTEXT_EXAMPLES:
			current_context = new ContextExample();
			current_context.setKEStoreKey( session_.getExampleOfWorksheet( worksheet_id_ ) );
			break;
		case CONTEXT_METHODS:
		case CONTEXT_PROBLEMS:
		case CONTEXT_THEORIES:
				try {
					current_context = context_formula_.initContext( type );
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			break;
		}

		System.out.println("current context:"+current_context);
		
		if( current_context == null )
			return new ContextUnknown();
		else
			return current_context;
	}
	
	/* (non-Javadoc)
	 * @see isac.wsdialog.IContextProvider#refineContext(isac.util.formulae.Context)
	 */
	public Context refineContext(Context context) {

		Context current_context = null;
		
		try {
			current_context = context_formula_.refineProblem( context );
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		if( current_context == null )
			return new ContextUnknown();
		else
			return current_context;
	}
	
	public void setContext(Context context) {
		CalcHead ch = null;
		
		try {
			if( context instanceof ContextTheory ) {
				System.out.println("In WSDialog::setContext() : Theory");
				context_formula_.setContext( (ContextTheory) context );
				return;
			} else 	if  ( context instanceof ContextMethod ) {
				System.out.println("In WSDialog::setContext() : Method");
				ch = context_formula_.setContext( (ContextMethod) context );
			} else 	if  ( context instanceof ContextProblem ) {
				System.out.println("In WSDialog::setContext() : Problem");
				ch = context_formula_.setContext( (ContextProblem) context );
			} else { 
				System.out.println("Can not set this context (Basis class, ContextExample or ContextUnknown)");
				return;
			}
		
			if( ch != null ) {
				doUIAction(new UIActionOnCalcElement(EUIElement.UI_SPECIFY_CHECK_CALCHEAD,
						EUIContext.UI_CONTEXT_CALCHEAD, user_language_, ch ));
			}
		
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		return;
	}
	
	/* (non-Javadoc)
	 * @see isac.wsdialog.IContextProvider#checkContext(isac.util.formulae.Context)
	 */
	public Context checkContext(Context context) {
		Context current_context = null;
	
		try {
			current_context = context_formula_.checkContext( context );
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		if( current_context == null )
			return new ContextUnknown();
		else
			return current_context;
	}

	/**
	 * @see isac.wsdialog.IContextProvider#getWorksheetState()
	 */
	public WorksheetState getWorksheetState() {
//		try {
//			WorksheetState wsstate = new WorksheetState( context_formula_.onCalcHead() );
//			return wsstate;
//		} catch (RemoteException e) {
//			e.printStackTrace();
//		} 
		
		return new WorksheetState( phase_ == DIALOGPHASE_SPECIFY );
	}

}
